<?php

/**
 * @file
 * Provides a simple time-based caching option for panel panes.
 */

// Plugin definition.
$plugin = array(
  'title' => t('Autocache'),
  'description' => t('Automatic caching.'),
  'cache get' => 'autocache_cache_get_cache',
  'cache set' => 'autocache_cache_set_cache',
  'cache clear' => 'autocache_cache_clear_cache',
  'cache clear pane' => 'autocache_cache_clear_pane_cache',
  'settings form' => 'autocache_cache_settings_form',
  'settings form validate' => 'autocache_cache_settings_form_validate',
  'settings form submit' => 'autocache_cache_settings_form_submit',
  'defaults' => array(
    'granularity' => 'none',
    'language' => 1,
    'language_content' => 1,
  ),
);

/**
 * Get cached content.
 */
function autocache_cache_get_cache($conf, $display, $args, $contexts, $pane = NULL) {
  if (!empty($conf['new'])) {
    $cid = autocache_cache_get_id($conf, $display, $args, $contexts, $pane);
    $cache = cache_get($cid, 'cache');
    if (!$cache) {
      return FALSE;
    }
    return $cache->data;
  }
}

/**
 * Set cached content.
 */
function autocache_cache_set_cache($conf, $content, $display, $args, $contexts, $pane = NULL) {
  if (!empty($conf['new'])) {
    $cid = autocache_cache_get_id($conf, $display, $args, $contexts, $pane);
    cache_set($cid, $content);
  }
}

/**
 * Clear cached content.
 *
 * @param stdClass $display
 *   The display object. If this object has a property named clear_pane, then
 *   that pane will be cleared.
 */
function autocache_cache_clear_cache($display) {
  $cid = 'rc';
  // Add some text if this is a pane, so that we don't clear all panes when
  // we clear a panel pane.
  if (isset($display->clear_pane)) {
    $cid .= ':p';
  }
  if (is_numeric($display->did) && $display->did) {
    $cid .= ':' . $display->did;
  }
  else {
    // Add the cache key if this is an in-code display.
    $cid .= ':' . $display->cache_key;
  }
  // If this is a mini panel then we have the owner property. Let's
  // use the machine name for those.
  if (isset($display->owner->name)) {
    $cid .= ':' . $display->owner->name;
  }
  // If we have a pane specified, then append that to the key as well.
  if (isset($display->clear_pane)) {
    $cid .= ':' . $display->clear_pane->pid;
  }
  
  cache_clear_all($cid, 'cache', TRUE);
}

/**
 * Figure out an id for our cache based upon input and settings.
 */
function autocache_cache_get_id($conf, $display, $args, $contexts, $pane) {
  
  $cid = array();
  $cid[] = 'ac';

  foreach ($display->context as $key => &$context) {
    $entity_type = is_array($context->type) ? (isset($context->type[2]) ? $context->type[2] : AUTOCACHE_NOBUNDLE) : $context->type; 
    $info = entity_get_info($entity_type);
    $cid[] = $info['base table'];
    $cid[] = isset($context->restrictions['type']) ? $context->restrictions['type'][0] : AUTOCACHE_NOBUNDLE;
    $cid[] = $context->argument;
  }

  $cid[] = 'display-' . $display->did;

  if ($pane) {
    $cid[] = 'pane-' . $pane->pid;    
  }

  $cid = implode('-', $cid);
  $cid = autocache_register_cache_key($cid);

  $id = $cid;

  if (!empty($display->context) && !empty($conf['substitute'])) {
    $id = ctools_context_keyword_substitute($id, array(), $display->context);
  }
  
  if (user_access('view pane admin links')) {
    $id .= ':adm';
  }

  switch ($conf['granularity']) {
    case 'args':
      foreach ($args as $arg) {
        // Arguments can be many things, but we only consider the ones that are
        // string values.
        if (is_string($arg)) {
          $id .= ':' . $arg;
        }
      }
      break;

    case 'context':
      if (!is_array($contexts)) {
        $contexts = array($contexts);
      }
      foreach ($contexts as $context) {
        if (isset($context->argument)) {
          $id .= ':' . $context->argument;
        }
      }
      break;
  }
  // Add cache for individual language.
  if ($conf['language']) {
    global $language;
    $id .= ':' . $language->language;
  }
  // Add cache for individual content language.
  if (!empty($conf['language_content'])) {
    global $language_content;
    $id .= ':' . $language_content->language;
  }

  $cache = new stdClass();
  
  $cache->conf = $conf;
  $cache->display = $display;
  $cache->args = $args;
  $cache->contexts = $contexts;
  $cache->pane = $pane;
  $cache->key = $id;

  return $cache->key;
}

/**
 * Provide a form for the cache settings.
 *
 * @param array $conf
 *   The current configuration.
 * @param stdClass $display
 *   The display object
 * @param string $pid
 *   The pane pid.
 */
function autocache_cache_settings_form(&$conf, $display, $pid) {
  $form = array();

  $form['granularity'] = array(
    '#title' => t('Granularity'),
    '#type' => 'select',
    '#options' => array(
      'args' => t('Arguments'),
      'context' => t('Context'),
      'none' => t('None'),
    ),
    '#description' => t('If "arguments" are selected, this content will be cached per individual argument to the entire display; if "contexts" are selected, this content will be cached per unique context in the pane or display; if "neither" there will be only one cache for this pane.'),
    '#default_value' => $conf['granularity'],
  );
  $form['language'] = array(
    '#title' => t('Cache per interface language'),
    '#type' => 'checkbox',
    '#default_value' => $conf['language'],
    '#description' => t('Select this if you want to cache content per language'),
  );
  $form['language_content'] = array(
    '#title' => t('Cache per content language'),
    '#type' => 'checkbox',
    '#default_value' => $conf['language_content'],
    '#description' => t('Select this if you want to cache content per language'),
  );
  return $form;
}

/**
 * Validates the cache settings.
 */
function autocache_cache_settings_form_validate($form, $values) {
  $args = func_get_args();
  $display = $form['display']['#value'];
  $pid = $form['pid']['#value'];
}

/**
 * Saves the cache settings.
 *
 * @param array $values
 *   The values to store.
 */
function autocache_cache_settings_form_submit(&$values) {
  $values['substitute'] = $values['advanced']['substitute'];
  // Add an indicator that this is stored with the new caching mechanism.
  $values['new'] = TRUE;
}
